--[[
	Copyright (C) 2006-2007 Nymbia

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
]]
local L = AceLibrary("AceLocale-2.2"):new("Quartz")
local BS = AceLibrary("Babble-Spell-2.2")
local sm = AceLibrary("SharedMedia-1.0")
local autoshotname = BS["Auto Shot"]
local resetspells = {
	[BS["Raptor Strike"]] = true,
	[BS["Heroic Strike"]] = true,
	[BS["Maul"]] = true,
	[BS["Slam"]] = true,
}

local unpack = unpack
local math_abs = math.abs

local Quartz = Quartz
if Quartz:HasModule('Swing') then
	return
end
local QuartzSwing = Quartz:NewModule('Swing')
local QuartzPlayer = Quartz:GetModule('Player')

local GetTime = GetTime

local swingbar, swingbar_width, swingstatusbar, db
local swingmode -- nil is none, 0 is meleeing, 1 is autoshooting
local starttime, duration

local BOOKTYPE_SPELL = BOOKTYPE_SPELL

local function OnUpdate()
	local perc = (GetTime() - starttime) / duration
	if perc > 1 then
		return swingbar:Hide()
	else
		swingstatusbar:SetValue(perc)
	end
end
local function OnHide()
	swingbar:SetScript('OnUpdate', nil)
	usingspell = nil
end
local function OnShow()
	swingbar:SetScript('OnUpdate', OnUpdate)
end

function QuartzSwing:OnInitialize()
	db = Quartz:AcquireDBNamespace("Swing")
	Quartz:RegisterDefaults("Swing", "profile", {
		barcolor = {1, 1, 1},
		swingalpha = 1,
		swingheight = 4,
		swingposition = L["Top"],
		swinggap = -4,
	})
end
function QuartzSwing:OnEnable()
	-- fired when autoattack is enabled/disabled.
	self:RegisterEvent("PLAYER_ENTER_COMBAT")
	self:RegisterEvent("PLAYER_LEAVE_COMBAT")
	-- fired when autoshot (or autowand) is enabled/disabled
	self:RegisterEvent("START_AUTOREPEAT_SPELL")
	self:RegisterEvent("STOP_AUTOREPEAT_SPELL")
	
	self:RegisterEvent("CHAT_MSG_COMBAT_SELF_HITS")
	self:RegisterEvent("CHAT_MSG_COMBAT_SELF_MISSES")
	
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	if not swingbar then
		swingbar = CreateFrame('Frame', 'QuartzSwingBar', UIParent)
		swingbar:SetFrameStrata('HIGH')
		swingbar:SetScript('OnShow', OnShow)
		swingbar:SetScript('OnHide', OnHide)
		
		swingstatusbar = CreateFrame("StatusBar", nil, swingbar)
		swingbar:Hide()
	end
	Quartz.ApplySettings()
end
function QuartzSwing:OnDisable()
	swingbar:Hide()
end
function QuartzSwing:PLAYER_ENTER_COMBAT()
	swingmode = 0
end
function QuartzSwing:PLAYER_LEAVE_COMBAT()
	swingmode = nil
	starttime = nil
	duration = nil
	swingbar:Hide()
end
function QuartzSwing:START_AUTOREPEAT_SPELL()
	swingmode = 1
end
function QuartzSwing:STOP_AUTOREPEAT_SPELL()
	swingmode = nil
	starttime = nil
	duration = nil
	swingbar:Hide()
end
do
	local critstr = '^'..COMBATHITCRITSELFOTHER:gsub("%.", "%%%."):gsub('%%s', '.-'):gsub('%%d', '(%%d+)')
	local hitstr = '^'..COMBATHITSELFOTHER:gsub("%.", "%%%."):gsub('%%s', '.-'):gsub('%%d', '(%%d+)')
	local function getdamage(msg)
		local d = msg:match(hitstr)
		if not d then
			d = msg:match(critstr)
			if not d then
				return
			end
			return tonumber(d) / 2.5
		end
		return tonumber(d)
	end
	function QuartzSwing:CHAT_MSG_COMBAT_SELF_HITS(msg)
		local low = UnitDamage('player')
		local damage = getdamage(msg)
		if damage then
			if duration then
				if damage >= (low * 0.8) or math_abs((starttime + duration) - GetTime()) <= 0.2 or (starttime + duration) < GetTime() then
					self:MeleeSwing()
				end
			else
				self:MeleeSwing()
			end
		end
	end
end
function QuartzSwing:CHAT_MSG_COMBAT_SELF_MISSES(msg)
	if duration and math_abs((starttime + duration) - GetTime()) <= 0.2 or (starttime + duration) < GetTime() then
		--close enough!
		self:MeleeSwing()
	end
end
function QuartzSwing:UNIT_SPELLCAST_SUCCEEDED(unit, spell)
	if swingmode == 0 then
		if resetspells[spell] then
			self:MeleeSwing()
		end
	elseif swingmode == 1 then
		if spell == autoshotname then
			self:Shoot()
		end
	end
end
function QuartzSwing:MeleeSwing()
	duration = UnitAttackSpeed('player')
	starttime = GetTime()
	swingbar:Show()
end
function QuartzSwing:Shoot()
	duration = UnitRangedDamage('player')
	starttime = GetTime()
	swingbar:Show()
end
function QuartzSwing:ApplySettings()
	if swingbar and Quartz:IsModuleActive('Swing') then
		local ldb = db.profile
		swingbar:ClearAllPoints()
		swingbar:SetHeight(ldb.swingheight)
		swingbar_width = QuartzCastBar:GetWidth() - 8
		swingbar:SetWidth(swingbar_width)
		swingbar:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16})
		swingbar:SetBackdropColor(0,0,0)
		swingbar:SetAlpha(ldb.swingalpha)
		swingbar:SetScale(QuartzPlayer.db.profile.scale)
		if ldb.swingposition == L["Bottom"] then
			swingbar:SetPoint("TOP", QuartzCastBar, "BOTTOM", 0, -1 * ldb.swinggap)
		else -- L["Top"]
			swingbar:SetPoint("BOTTOM", QuartzCastBar, "TOP", 0, ldb.swinggap)
		end
		
		swingstatusbar:SetAllPoints(swingbar)
		swingstatusbar:SetStatusBarTexture(sm:Fetch('statusbar', Quartz:GetModule('Player').db.profile.texture))
		swingstatusbar:SetStatusBarColor(unpack(ldb.barcolor))
		swingstatusbar:SetMinMaxValues(0, 1)
	end
end
do
	local function set(field, value)
		db.profile[field] = value
		Quartz.ApplySettings()
	end
	local function get(field)
		return db.profile[field]
	end
	local function setcolor(field, ...)
		db.profile[field] = {...}
		Quartz.ApplySettings()
	end
	local function getcolor(field)
		return unpack(db.profile[field])
	end
	Quartz.options.args.Swing = {
		type = 'group',
		name = L["Swing"],
		desc = L["Swing"],
		order = 600,
		args = {
			toggle = {
				type = 'toggle',
				name = L["Enable"],
				desc = L["Enable"],
				get = function()
					return Quartz:IsModuleActive('Swing')
				end,
				set = function(v)
					Quartz:ToggleModuleActive('Swing', v)
				end,
				order = 100,
			},
			barcolor = {
				type = 'color',
				name = L["Bar Color"],
				desc = L["Set the color of the swing timer bar"],
				get = getcolor,
				set = setcolor,
				passValue = 'barcolor',
				order = 103,
			},
			swingheight = {
				type = 'range',
				name = L["Height"],
				desc = L["Set the height of the swing timer bar"],
				min = 1,
				max = 20,
				step = 1,
				get = get,
				set = set,
				passValue = 'swingheight',
				order = 104,
			},
			swingalpha = {
				type = 'range',
				name = L["Alpha"],
				desc = L["Set the alpha of the swing timer bar"],
				min = 0.05,
				max = 1,
				step = 0.05,
				isPercent = true,
				get = get,
				set = set,
				passValue = 'swingalpha',
				order = 105,
			},
			swingposition = {
				type = 'text',
				name = L["Bar Position"],
				desc = L["Set the position of the swing timer bar"],
				get = get,
				set = set,
				passValue = 'swingposition',
				validate = {L["Top"], L["Bottom"]},
				order = 106,
			},
			swinggap = {
				type = 'range',
				name = L["Gap"],
				desc = L["Tweak the distance of the swing timer bar from the cast bar"],
				min = -10,
				max = 10,
				step = 1,
				get = get,
				set = set,
				passValue = 'swinggap',
				order = 107,
			},
		},
	}
end